/*
STM32G431 WeAct Duino program for HC-SR04
The duration of the sensor echo signal is measured by generating an
external interrupt every time the echo signal changes state.
Initial Creation 3/16/24     -- jkl
!!!!!!!!!!!! HC-SR04 Echo pin must be hooked to a 5V tolerant input !!!!!!!!!!!
HardWare Serial must be used.

Adapted from:
Arduino Uno HC-SR04 ultrasonic range finder program using interrupts 
Author:  Steve Garratt 
Date:    19/10/13
*/

// Order is Rx,Tx
HardwareSerial mySerial(23,24);   // Usart 1

// 18 is a good 5V tolerant input. Can use 17 for the output trigger.
const int trigPin = 17;
const int echoPin = 18; 

volatile long echo_start = 0;                         // Records start of echo pulse 
volatile long echo_end = 0;                           // Records end of echo pulse

void setup() 
{
  pinMode(trigPin, OUTPUT);                           // Trigger pin set to output
  pinMode(echoPin, INPUT);                            // Echo pin set to input
  attachInterrupt(echoPin, echo_interrupt, CHANGE);  // Attach interrupt to the sensor echo pin
  mySerial.begin(115200);                             // Initialise the serial monitor output
}

void loop()
{
  long echo_duration = 0;                      // Duration - difference between end and start
  int distance;

  	// Write a pulse to the HC-SR04 Trigger Pin
  echo_end = 0;                     // becomes non-zero when a pulse happens
	digitalWrite(trigPin, LOW);
	delayMicroseconds(2);
	digitalWrite(trigPin, HIGH);
	delayMicroseconds(10);
	digitalWrite(trigPin, LOW);

  // Wait until echo_end is non-zero (set in ISR)
  // Measure the response from the HC-SR04 Echo Pin
  while (echo_end == 0){}    // this is blocking. Could be non-blocking with a change of logic.

  echo_duration = echo_end - echo_start;        // Calculate the pulse duration
	
	// Determine distance from duration
	// Use 343 metres per second as speed of sound
  distance= echo_duration*0.034/2;
  mySerial.print("Distance: "); 
	mySerial.print(distance);
	mySerial.println(" cm");

	delay(500);
}
// --------------------------
// echo_interrupt() External interrupt from HC-SR04 echo signal. 
// Called every time the echo signal changes state.
// --------------------------
void echo_interrupt()
{
  switch (digitalRead(echoPin))                     // Test to see if the signal is high or low
  {
    case HIGH:                                      // High so must be the start of the echo pulse
      echo_start = micros();                        // Save the start time
      break;
      
    case LOW:                                       // Low so must be the end of the echo pulse
      echo_end = micros();                          // Save the end time
      break;
  }
}
